@@ -1,92 +1,72 @@
 ACF <- NULL
 
-#' Adaptive Trend and Cycle Analysis for Time Series
+#' Adaptive Trend and Cycle Analysis for Time Series (universal version)
 #'
 #' @description
 #' Performs adaptive trend estimation, cycle detection, Fourier harmonic
 #' selection, bootstrap confidence intervals, change points detection, and
 #' rolling-origin forecasting. Supports LOESS, GAM, and GAMM models, and
-#' automatically handles irregular sampling using the Lomb–Scargle periodogram.
+#' handles irregular sampling using the Lomb-Scargle periodogram.
 #'
-#' @importFrom mgcv gam
-#' @importFrom lomb lsp
-#' @importFrom stats coef Box.test acf as.formula gaussian lm loess median predict
-#' @importFrom stats qchisq quantile setNames spec.pgram stl ts
-#' @importFrom changepoint cpts cpt.meanvar
-#' @importFrom ggplot2 ggplot aes geom_line geom_point labs geom_ribbon geom_vline
-#' @importFrom ggplot2 theme_minimal annotate geom_col geom_hline scale_color_manual
-#' @importFrom gridExtra arrangeGrob grid.arrange
-#' @importFrom magrittr %>%
-#' @importFrom dplyr mutate
-#' @importFrom tseries adf.test kpss.test
-#' @importFrom utils tail
+#' Works with:
+#' - dates_type = "date"   : Date (daily/weekly/monthly data)
+#' - dates_type = "posix"  : POSIXct (sub-daily, wearable, EEG, sensors)
+#' - dates_type = "numeric": numeric time (spike trains, simulations)
 #'
 #' @param signal Numeric vector of observed values.
-#' @param dates Date vector of the same length as `signal`.
+#' @param dates Vector of time indices (Date, POSIXct, or numeric).
+#' @param dates_type "date", "posix", or "numeric".
 #' @param normalize Logical; if TRUE, Z score normalization is applied.
-#' @param trendmethod Character; `"loess"` or `"gam"`.
+#' @param trendmethod "loess" or "gam".
 #' @param usefourier Logical; whether to include Fourier harmonics.
 #' @param fourierK Integer; fixed number of harmonics if auto selection disabled.
 #' @param auto_fourier_select Logical; if TRUE, selects K via AICc/BIC.
-#' @param fourier_selection_criterion `"AICc"` or `"BIC"`.
+#' @param fourier_selection_criterion "AICc" or "BIC".
 #' @param fourierK_max Maximum K to consider during selection.
-#' @param cimethod `"model"`, `"bootstrapiid"`, or `"bootstrapmbb"`.
+#' @param cimethod "model", "bootstrapiid", or "bootstrapmbb".
 #' @param nboot Number of bootstrap samples.
 #' @param blocksize Block size for MBB bootstrap.
-#' @param seasonalfrequency Seasonal frequency for STL (regular sampling).
+#' @param seasonalfrequency Seasonal frequency for STL (only for dates_type="date").
 #' @param stlrobust Logical; robust STL decomposition.
 #' @param specspans Smoothing spans for spectral estimation.
 #' @param auto_seasonality Logical; if TRUE, uses dominant period.
 #' @param lagmax Maximum lag for ACF and Ljung Box tests.
-#' @param loess_span_mode `"auto_aicc"`, `"auto_gcv"`, `"cv"`, `"fixed"`.
+#' @param loess_span_mode "auto_aicc", "auto_gcv", "cv", "fixed".
 #' @param loess_span_fixed Numeric; fixed LOESS span.
 #' @param loess_span_grid Grid of spans for CV.
 #' @param loess_cv_k Number of folds for blocked CV.
-#' @param blocklength_mode `"auto_pwsd"`, `"heuristic"`, `"fixed"`.
+#' @param blocklength_mode "auto_pwsd", "heuristic", "fixed".
 #' @param blocklength_fixed Fixed block length.
 #' @param robust Logical; robust LOESS or robust GAM family.
 #' @param use_gamm Logical; fit GAMM instead of GAM.
 #' @param group_var Character; grouping variable for random intercepts.
 #' @param group_values Optional vector to attach as grouping variable.
-#' @param random_effect Optional random effects list for `mgcv::gamm`.
-#' @param cor_struct `"none"`, `"ar1"`, `"arma"`.
+#' @param random_effect Optional random effects list for mgcv::gamm.
+#' @param cor_struct "none", "ar1", "arma".
 #' @param arma_p,arma_q ARMA orders.
 #' @param forecast_holdout_h Holdout horizon for forecasting.
-#' @param forecast_origin_mode `"expanding"` or `"sliding"`.
+#' @param forecast_origin_mode "expanding" or "sliding".
 #' @param train_window Training window for sliding origin.
 #' @param forecast_lock_K Logical; lock Fourier K across origins.
 #'
-#' @return A list containing:
-#' \itemize{
-#'   \item Trend estimates
-#'   \item Confidence intervals
-#'   \item Residuals and diagnostics
-#'   \item Fourier selection results
-#'   \item Change-point locations
-#'   \item Spectral analysis
-#'   \item Forecast results (if enabled)
-#'   \item ggplot2 objects for visualization
-#' }
+#' @return A list with:
+#' - Data (with PlotDate, timenum, Trend, CI, Outlier)
+#' - Trend
+#' - CI (lower, upper)
+#' - Residuals
+#' - Fourier (K)
+#' - ChangePoints (in PlotDate scale)
+#' - Spectrum
+#' - Plot (Trend, Spectrum)
 #'
-#' @examples
-#' set.seed(1)
-#'
-#' dates <- as.Date("2020-01-01") + 1:50
-#' signal <- sin(2*pi*(1:50)/10) + rnorm(50, 0, 0.1)
-#'
-#' res <- adaptive_cycle_trend_analysis(
-#'   signal = signal,
-#'   dates = dates,
-#'   trendmethod = "loess",
-#'   usefourier = FALSE
-#' )
-#'
-#' plot(res$Plot$Trend)
+#' @importFrom stats Box.test acf as.formula coef gaussian lm loess median predict quantile setNames spec.pgram
+#' @importFrom lomb lsp
 #'
 #' @export
 adaptive_cycle_trend_analysis <- function(
     signal,
     dates,
+    dates_type = c("date", "posix", "numeric"),
     normalize = FALSE,
     trendmethod = c("loess", "gam"),
     usefourier = FALSE,
@@ -130,10 +110,10 @@ adaptive_cycle_trend_analysis <- function(
   forecast_origin_mode <- match.arg(forecast_origin_mode)
   fourier_selection_criterion <- match.arg(fourier_selection_criterion)
   cor_struct <- match.arg(cor_struct)
+  dates_type <- match.arg(dates_type)
 
-  # ---- Validate ----
+  # ---- Validate basic ----
   if (!is.numeric(signal)) stop("signal must be numeric.")
-  if (!inherits(dates, "Date")) stop("dates must be of class Date.")
   if (length(signal) != length(dates)) stop("signal and dates must have same length.")
   if (usefourier && fourierK < 0) stop("fourierK must be >= 0.")
   N <- length(signal)
@@ -155,25 +135,50 @@ adaptive_cycle_trend_analysis <- function(
     message("Using original scale.")
   }
 
-  # ---- Regularity ----
-  datediff <- diff(as.numeric(dates))
-  irregular <- if (length(datediff) > 0) any(abs(datediff - median(datediff)) > 1e-6) else FALSE
-  message(ifelse(irregular, "Irregular dates detected: Lomb Scargle + LOESS/GAM(M)",
-                 "Regular dates: STL + LOESS/GAM(M)"))
+  # ---- Time handling: timenum + PlotDate ----
+  if (dates_type == "date") {
+    if (!inherits(dates, "Date"))
+      stop("dates must be of class Date when dates_type='date'.")
+    timenum <- as.numeric(dates)          # days since origin
+    PlotDate <- dates
+    use_stl <- TRUE
+  } else if (dates_type == "posix") {
+    if (!inherits(dates, "POSIXct"))
+      stop("dates must be POSIXct when dates_type='posix'.")
+    timenum <- as.numeric(difftime(dates, min(dates), units = "secs"))
+    PlotDate <- dates
+    use_stl <- FALSE
+    auto_seasonality <- FALSE
+  } else { # numeric
+    if (!is.numeric(dates))
+      stop("dates must be numeric when dates_type='numeric'.")
+    timenum <- dates
+    PlotDate <- timenum
+    use_stl <- FALSE
+    auto_seasonality <- FALSE
+  }
 
-  # ---- Data & time index ----
-  df <- data.frame(Date = dates, Value = signal)
-  timenum <- as.numeric(df$Date)
-  df$timenum <- timenum
-  avgintervaldays <- if (length(datediff)) mean(datediff) else 1
+  # ---- Regularity ----
+  datediff <- diff(timenum)
+  irregular <- if (length(datediff) > 0) any(abs(datediff - median(datediff)) > 1e-6) else FALSE
+  message(ifelse(irregular,
+                 "Irregular time index: Lomb-Scargle + LOESS/GAM(M)",
+                 "Regular time index: STL (if dates_type='date') + LOESS/GAM(M)"))
+
+  avginterval <- if (length(datediff)) mean(datediff) else 1
+
+  # ---- Data frame ----
+  df <- data.frame(
+    PlotDate = PlotDate,
+    Value    = signal,
+    timenum  = timenum
+  )
 
   # ---- Attach/convert group_var ----
   if (use_gamm && !is.null(group_var)) {
-    # Ensure group_var is a proper character scalar
     if (!is.character(group_var) || length(group_var) != 1L || nchar(group_var) == 0L) {
       stop("`group_var` must be a non-empty character scalar naming a column.")
     }
-    # Attach group_values if df lacks the column
     if (!group_var %in% names(df)) {
       if (!is.null(group_values)) {
         if (length(group_values) != nrow(df)) {
@@ -184,11 +189,9 @@ adaptive_cycle_trend_analysis <- function(
         message(sprintf("Attached grouping column '%s' from `group_values`.", group_var))
       }
     }
-    # Column must exist now
     if (!group_var %in% names(df)) {
       stop(sprintf("group_var '%s' not found in the data.", group_var))
     }
-    # Convert to factor if needed
     if (!is.factor(df[[group_var]])) {
       df[[group_var]] <- as.factor(df[[group_var]])
       message(sprintf("Converted group_var '%s' to factor (%d levels).",
@@ -196,51 +199,58 @@ adaptive_cycle_trend_analysis <- function(
     }
   }
 
-  # ---- Dominant period estimation (fixed irregular branch) ----
-  estimate_dominant <- function(x, dates, irregular, specspans, avgintervaldays) {
-    if (!irregular) {
+  # ---- Dominant period estimation (unified) ----
+  estimate_dominant <- function(x, timenum, irregular, specspans, avginterval) {
+    if (!irregular && dates_type == "date") {
       spec <- spec.pgram(x, spans = specspans, log = "no", taper = 0.1, detrend = TRUE)
       freq <- spec$freq; power <- spec$spec
       if (length(power) > 0) {
         ix <- which.max(power); dfreq <- freq[ix]
-        list(freq = dfreq,
-             period_samples = if (!is.na(dfreq) && dfreq > 0) 1 / dfreq else NA_real_,
-             period_days = if (!is.na(dfreq) && dfreq > 0) (1 / dfreq) * avgintervaldays else NA_real_)
+        list(
+          freq = dfreq,
+          period_samples = if (!is.na(dfreq) && dfreq > 0) 1 / dfreq else NA_real_,
+          period_units   = if (!is.na(dfreq) && dfreq > 0) (1 / dfreq) * avginterval else NA_real_
+        )
       } else {
-        list(freq = NA_real_, period_samples = NA_real_, period_days = NA_real_)
+        list(freq = NA_real_, period_samples = NA_real_, period_units = NA_real_)
       }
     } else {
-      tnum <- as.numeric(dates - min(dates))
+      tnum <- timenum - min(timenum)
       lombres <- try(lsp(x, tnum, type = "frequency", plot = FALSE), silent = TRUE)
       if (inherits(lombres, "try-error") ||
           is.null(lombres$peak.at) || length(lombres$peak.at) < 2 ||
           any(!is.finite(lombres$peak.at))) {
-        return(list(freq = NA_real_, period_samples = NA_real_, period_days = NA_real_))
+        return(list(freq = NA_real_, period_samples = NA_real_, period_units = NA_real_))
       }
       dfreq <- lombres$peak.at[1]
-      p_days <- lombres$peak.at[2]
+      p_units <- lombres$peak.at[2]
       list(
         freq = dfreq,
         period_samples = if (!is.na(dfreq) && dfreq > 0) 1 / dfreq else NA_real_,
-        period_days = p_days
+        period_units   = p_units
       )
     }
   }
-  dom <- estimate_dominant(signal, dates, irregular, specspans, avgintervaldays)
+
+  dom <- estimate_dominant(signal, timenum, irregular, specspans, avginterval)
 
   # ---- Seasonality from dominant period ----
-  if (auto_seasonality && !is.na(dom$period_samples) && dom$period_samples > 1) {
+  if (dates_type == "date" && auto_seasonality &&
+      !is.na(dom$period_samples) && dom$period_samples > 1) {
     seasonalfrequency <- max(2L, round(dom$period_samples))
     message(sprintf("Auto seasonality: dominant period approx %.2f samples -> seasonalfrequency=%d",
                     dom$period_samples, seasonalfrequency))
-  } else {
+  } else if (dates_type == "date") {
     message(sprintf("Using provided seasonalfrequency=%d", seasonalfrequency))
+  } else {
+    message("No STL/seasonality for dates_type != 'date'.")
   }
-  perioddays <- if (!is.na(dom$period_days)) dom$period_days else seasonalfrequency * avgintervaldays
 
-  # ---- Fourier design ----
-  makefourier <- function(timenum, perioddays, K) {
-    phi <- 2 * pi * timenum / perioddays
+  period_units <- if (!is.na(dom$period_units)) dom$period_units else seasonalfrequency * avginterval
+
+  # ---- Fourier design (in units of timenum) ----
+  makefourier <- function(timenum, period_units, K) {
+    phi <- 2 * pi * timenum / period_units
     out <- data.frame(row.names = seq_along(timenum))
     if (K >= 1) for (k in seq_len(K)) {
       out[[paste0("sin", k)]] <- sin(k * phi)
@@ -248,8 +258,13 @@ adaptive_cycle_trend_analysis <- function(
     }
     out
   }
+
   maxK <- if (usefourier) max(fourierK, fourierK_max) else 0
-  full_fourier_design <- if (usefourier && maxK > 0) makefourier(timenum, perioddays, maxK) else NULL
+  full_fourier_design <- if (usefourier && maxK > 0 && is.finite(period_units) && period_units > 0) {
+    makefourier(timenum, period_units, maxK)
+  } else {
+    NULL
+  }
 
   # ---- Helper: Blocked k-fold CV for LOESS span ----
   blocked_loess_cv <- function(y, x, spans, k = 5, robust = TRUE) {
@@ -322,7 +337,6 @@ adaptive_cycle_trend_analysis <- function(
     # GAM or GAMM
     family_for_fit <- if (use_gamm) gaussian() else if (robust) mgcv::scat() else gaussian()
     if (use_gamm) {
-      # Construct random and correlation structures
       rand_effect <- if (!is.null(random_effect)) random_effect else {
         if (!is.null(group_var)) setNames(list(~1), group_var) else NULL
       }
@@ -345,13 +359,11 @@ adaptive_cycle_trend_analysis <- function(
         random = rand_effect,
         correlation = correlation
       )
-      # In-sample predictions including random effects if available
       base_trend <- try(as.numeric(stats::predict(gammfit$lme,
                                                   newdata = df,
                                                   level = if (!is.null(group_var) || !is.null(random_effect)) 1 else 0)),
                         silent = TRUE)
       if (inherits(base_trend, "try-error") || any(!is.finite(base_trend))) {
-        # Fallback to fixed-effect smooth only
         base_trend <- as.numeric(predict(gammfit$gam, newdata = df))
       }
       trendobj <- gammfit
@@ -362,8 +374,8 @@ adaptive_cycle_trend_analysis <- function(
                            if (cor_struct != "none") paste0(", cor=", cor_struct) else "",
                            ")")
     } else {
-      gamfit <- gam(Value ~ s(timenum, bs = "cs"), data = df, method = "REML",
-                    family = family_for_fit)
+      gamfit <- mgcv::gam(Value ~ s(timenum, bs = "cs"), data = df, method = "REML",
+                          family = family_for_fit)
       base_trend <- as.numeric(predict(gamfit, newdata = df))
       trendobj <- gamfit
       edf <- tryCatch(sum(summary(gamfit)$s.table[, "edf"]), error = function(e) NA)
@@ -381,7 +393,6 @@ adaptive_cycle_trend_analysis <- function(
     resid_base <- df$Value - base_trend
     crit_vals <- rep(Inf, fourierK_max + 1)   # index 1 corresponds to K=0
     fit_list <- vector("list", length = fourierK_max + 1)
-    # K = 0 case (no Fourier)
     crit_vals[1] <- {
       rss <- sum((resid_base - mean(resid_base))^2, na.rm = TRUE)
       n <- sum(is.finite(resid_base))
@@ -437,7 +448,8 @@ adaptive_cycle_trend_analysis <- function(
   # ---- Final Trend (base + Fourier) ----
   df$Trend <- base_trend + df$FourierComponent
   if (usefourier) {
-    trendlabel <- sprintf("%s + Fourier (K=%d; period approx %.1f days)", trendlabel, selectedK, perioddays)
+    trendlabel <- sprintf("%s + Fourier (K=%d; period approx %.2f time units)",
+                          trendlabel, selectedK, period_units)
   }
 
   # ---- Residuals & diagnostics ----
@@ -454,7 +466,6 @@ adaptive_cycle_trend_analysis <- function(
   }
   cilower <- ciupper <- rep(NA_real_, N)
 
-  # helper for bootstrap refits (lock selectedK)
   refitpredict <- function(yvec) {
     if (trendmethod == "loess") {
       fit1 <- loess(yvec ~ timenum, span = if (exists("span_opt")) span_opt else 0.3,
@@ -489,10 +500,10 @@ adaptive_cycle_trend_analysis <- function(
           tr1 <- as.numeric(predict(fit$gam, newdata = df))
         }
       } else {
-        fit <- gam(Value ~ s(timenum, bs = "cs"),
-                   data = transform(df, Value = yvec),
-                   method = "REML",
-                   family = if (robust) mgcv::scat() else gaussian())
+        fit <- mgcv::gam(Value ~ s(timenum, bs = "cs"),
+                         data = transform(df, Value = yvec),
+                         method = "REML",
+                         family = if (robust) mgcv::scat() else gaussian())
         tr1 <- as.numeric(predict(fit, newdata = df))
       }
     }
@@ -507,7 +518,6 @@ adaptive_cycle_trend_analysis <- function(
   }
 
   if (cimethod == "model") {
-    # Model-based intervals implemented only for base LOESS/GAM without Fourier
     if (trendmethod == "loess" && (!usefourier || selectedK == 0)) {
       pred <- predict(trendobj, newdata = data.frame(timenum = timenum), se = TRUE)
       ciupper <- pred$fit + 1.96 * pred$se.fit
@@ -605,30 +615,31 @@ adaptive_cycle_trend_analysis <- function(
   adfinterpret  <- if (adftest$p.value < 0.05) "ADF: Reject H0 -> likely stationary" else "ADF: Fail to reject H0 -> likely non stationary"
   kpssinterpret <- if (kpsstest$p.value < 0.05) "KPSS: Reject H0 -> likely non stationary" else "KPSS: Fail to reject H0 -> likely stationary"
 
-  cpobj    <- cpt.meanvar(df$Trend, method = "PELT", penalty = "BIC")
-  cppoints <- cpts(cpobj)
-  cpdates  <- if (length(cppoints)) df$Date[cppoints] else as.Date(character())
+  cpobj    <- changepoint::cpt.meanvar(df$Trend, method = "PELT", penalty = "BIC")
+  cppoints <- changepoint::cpts(cpobj)
+  cpdates  <- if (length(cppoints)) df$PlotDate[cppoints] else PlotDate[0]
 
   # ---- Spectra for plotting ----
   spectrumdf <- significantspectrum <- data.frame()
   significantperiods <- numeric(0)
-  dominantperioddays <- dom$period_days
+  dominantperiodunits <- dom$period_units
   dominantfrequency  <- dom$freq
-  if (!irregular) {
-    tsdata <- ts(signal, frequency = seasonalfrequency)
-    decomp <- stl(tsdata, s.window = "periodic", robust = stlrobust)
+
+  if (use_stl && !irregular && dates_type == "date") {
+    tsdata <- stats::ts(signal, frequency = seasonalfrequency)
+    decomp <- stats::stl(tsdata, s.window = "periodic", robust = stlrobust)
     df$Seasonal <- as.numeric(decomp$time.series[, "seasonal"])
-    spec  <- spec.pgram(signal, spans = specspans, log = "no", taper = 0.1, detrend = TRUE)
+    spec  <- stats::spec.pgram(signal, spans = specspans, log = "no", taper = 0.1, detrend = TRUE)
     freq  <- spec$freq; power <- spec$spec
     spectrumdf <- data.frame(Frequency = freq, Power = power)
     k      <- spec$df; alpha <- 0.01
-    upper  <- power * qchisq(1 - alpha/2, 2 * k) / (2 * k)
+    upper  <- power * stats::qchisq(1 - alpha/2, 2 * k) / (2 * k)
     sigix  <- which(power > upper)
     significantspectrum <- spectrumdf[sigix, , drop = FALSE]
-    significantperiods <- round(1 / significantspectrum$Frequency * avgintervaldays, 2)
+    significantperiods <- round(1 / significantspectrum$Frequency * avginterval, 2)
   } else {
-    tnum <- as.numeric(df$Date - min(df$Date))
-    lombres  <- lsp(signal, tnum, type = "frequency", plot = FALSE)
+    tnum <- timenum - min(timenum)
+    lombres  <- lomb::lsp(signal, tnum, type = "frequency", plot = FALSE)
     lombdf   <- data.frame(Frequency = lombres$scanned, Power = lombres$power)
     spectrumdf <- lombdf
     signiflevel <- lombres$sig.level
@@ -638,26 +649,26 @@ adaptive_cycle_trend_analysis <- function(
   }
 
   # ---- Plots ----
-  ptrend <- ggplot(df, aes(x = Date)) +
-    geom_line(aes(y = Value), color = "steelblue", linewidth = 0.8) +
-    geom_line(aes(y = Trend), color = "darkred", linewidth = 0.9) +
-    geom_ribbon(aes(ymin = cilower, ymax = ciupper),
-                fill = "grey80", alpha = 0.35) +
-    geom_point(data = subset(df, Outlier),
-               aes(y = Trend), color = "orange", size = 2) +
-    { if (length(cpdates)) geom_vline(xintercept = cpdates,
-                                      linetype = "dotted", color = "orange") } +
-    labs(title = sprintf("Trend (%s) with 95%% CI (%s)",
-                         trendlabel, cimethod),
-         y = "Signal", x = "Date") +
-    theme_minimal()
+  ptrend <- ggplot2::ggplot(df, ggplot2::aes(x = PlotDate)) +
+    ggplot2::geom_line(ggplot2::aes(y = Value), color = "steelblue", linewidth = 0.8) +
+    ggplot2::geom_line(ggplot2::aes(y = Trend), color = "darkred", linewidth = 0.9) +
+    ggplot2::geom_ribbon(ggplot2::aes(ymin = cilower, ymax = ciupper),
+                         fill = "grey80", alpha = 0.35) +
+    ggplot2::geom_point(data = subset(df, Outlier),
+                        ggplot2::aes(y = Trend), color = "orange", size = 2) +
+    { if (length(cpdates)) ggplot2::geom_vline(xintercept = cpdates,
+                                               linetype = "dotted", color = "orange") } +
+    ggplot2::labs(title = sprintf("Trend (%s) with 95%% CI (%s)",
+                                  trendlabel, cimethod),
+                  y = "Signal", x = "Time") +
+    ggplot2::theme_minimal()
 
-  pperiod <- ggplot(spectrumdf, aes(x = Frequency, y = Power)) +
-    geom_line(color = "black", linewidth = 0.6) +
-    geom_point(data = significantspectrum,
-               aes(x = Frequency, y = Power),
-               color = "red", size = 1.6) +
-    theme_minimal()
+  pperiod <- ggplot2::ggplot(spectrumdf, ggplot2::aes(x = Frequency, y = Power)) +
+    ggplot2::geom_line(color = "black", linewidth = 0.6) +
+    ggplot2::geom_point(data = significantspectrum,
+                        ggplot2::aes(x = Frequency, y = Power),
+                        color = "red", size = 1.6) +
+    ggplot2::theme_minimal()
 
   # ---- Return ----
   list(
@@ -665,10 +676,16 @@ adaptive_cycle_trend_analysis <- function(
     Trend = df$Trend,
     CI = list(lower = cilower, upper = ciupper),
     Residuals = residuals,
-    Fourier = list(K = selectedK),
+    Fourier = list(K = selectedK,
+                   dominant_period = dominantperiodunits,
+                   dominant_frequency = dominantfrequency),
     ChangePoints = cpdates,
     Spectrum = spectrumdf,
-    Plot = list(Trend = ptrend, Spectrum = pperiod)
+    Plot = list(Trend = ptrend, Spectrum = pperiod),
+    Tests = list(
+      ADF = list(statistic = adftest$statistic, p.value = adftest$p.value, interpretation = adfinterpret),
+      KPSS = list(statistic = kpsstest$statistic, p.value = kpsstest$p.value, interpretation = kpssinterpret),
+      LjungBox = list(statistic = ljungboxtest$statistic, p.value = ljungboxtest$p.value)
+    )
   )
 }
-
